import Jed from 'jed'
import getLocaleData from './getLocaleData'

const transatorMap = {}
let translator

/**
 *
 */
export function gettext(message) {
  return translator.gettext(message)
}

/**
 *
 */
export function ngettext(msg1, msg2, n) {
  return translator.ngettext(msg1, msg2, n)
}

/**
 *
 */
export function dgettext(domain, message) {
  return transatorMap[domain].gettext(message)
}

/**
 *
 */
export function dngettext($domain, msg1, msg2, n) {
  return transatorMap[domain].ngettext(msg1, msg2, n)
}

/**
 *
 */
export async function initTranslate(domains = "webpack") {
  const isArray = Array.isArray(domains)
  if (!isArray && typeof domains !== 'string') {
    console.error('The translation domain must be a string or array of strings.')
  }
  if (!isArray) {
    domains = [domains]
  }

  const localeData = await getLocaleData()

  for (let i = 0; i < domains.length; i++) {
    const domain = domains[i]
    const data = await getDomainData(localeData, domain)
    transatorMap[domain] = new Jed(data)
  }

  translator = transatorMap[domains[0]]

  return true
}

/**
 *
 */
async function getDomainData(localeData, domain) {
  const {fallbacks, native, preferences} = await getLocaleData()
  const {languages, locales} = preferences

  for (let i = 0; i < locales.length; i++) {
    const locale = locales[i]

    try {
      return await getPoData(locale, domain)
    } catch (e) {
      console.log(`missed '${locale}'`)
    }
  }

  for (let i = 0; i < languages.length; i++) {
    const language = language[i]

    if (!fallbacks.hasOwnProperty(language)) {
      continue;
    }
    const locale = fallbacks[language]

    try {
      return await getPoData(locale, domain)
    } catch (e) {
      console.log(`missed fallback '${locale}'`)
    }
  }

  // At last try the native locale.
  return await getPoData(native, domain)
}

/**
 * Webpack can use that there is only a single ${locale} variable and it will turn that into a
 * regular expression with '*' to load all of the locales.  Webpack can resolve the ${LOCALES_DIR}
 * constant during build time so that it doesn't appear as a variable during runtime.
 */
async function getPoData(locale, domain) {
  return await import(`${LOCALES_DIR}${locale}/LC_MESSAGES/${domain}.po`)
}
